@@ -1,7 +1,8 @@
 //! Provides functionality for handling HTTP requests.
 
 use crate::http::address::Address;
-use crate::http::headers::{RequestHeader, RequestHeaderMap};
+use crate::http::cookie::Cookie;
+use crate::http::headers::{HeaderType, Headers};
 use crate::http::method::Method;
 use crate::stream::Stream;
 
@@ -22,8 +23,8 @@ pub struct Request {
     pub query: String,
     /// The HTTP version of the request.
     pub version: String,
-    /// A map of headers included in the request.
-    pub headers: RequestHeaderMap,
+    /// A list of headers included in the request.
+    pub headers: Headers,
     /// The request body, if supplied.
     pub content: Option<Vec<u8>>,
     /// The address from which the request came
@@ -99,6 +100,29 @@ impl Request {
         Self::from_stream_inner(stream, address, first_buf[0])
     }
 
+    /// Get the cookies from the request.
+    pub fn get_cookies(&self) -> Vec<Cookie> {
+        self.headers
+            .get(HeaderType::Cookie)
+            .map(|cookies| {
+                cookies
+                    .split(';')
+                    .filter_map(|cookie| {
+                        let (k, v) = cookie.split_once('=')?;
+                        Some(Cookie::new(k.trim(), v.trim()))
+                    })
+                    .collect()
+            })
+            .unwrap_or_default()
+    }
+
+    /// Attempts to get a specific cookie from the request.
+    pub fn get_cookie(&self, name: impl AsRef<str>) -> Option<Cookie> {
+        self.get_cookies()
+            .into_iter()
+            .find(|cookie| cookie.name == name.as_ref())
+    }
+
     /// Attempts to read and parse one HTTP request from the given reader.
     fn from_stream_inner<T>(
         stream: &mut T,
@@ -137,7 +161,7 @@ impl Request {
         let uri = uri_iter.next().unwrap().to_string();
         let query = uri_iter.next().unwrap_or("").to_string();
 
-        let mut headers = RequestHeaderMap::new();
+        let mut headers = Headers::new();
 
         loop {
             let mut line_buf: Vec<u8> = Vec::with_capacity(256);
@@ -152,13 +176,12 @@ impl Request {
                 safe_assert(line.len() >= 2)?;
                 let line_without_crlf = &line[0..line.len() - 2];
                 let mut line_parts = line_without_crlf.splitn(2, ':');
-                headers.insert(
-                    RequestHeader::from(line_parts.next().to_error(RequestError::Request)?),
+                headers.add(
+                    HeaderType::from(line_parts.next().to_error(RequestError::Request)?),
                     line_parts
                         .next()
                         .to_error(RequestError::Request)?
-                        .trim_start()
-                        .to_string(),
+                        .trim_start(),
                 );
             }
         }
@@ -166,7 +189,7 @@ impl Request {
         let address =
             Address::from_headers(&headers, address).map_err(|_| RequestError::Request)?;
 
-        if let Some(content_length) = headers.get(&RequestHeader::ContentLength) {
+        if let Some(content_length) = headers.get(&HeaderType::ContentLength) {
             let content_length: usize =
                 content_length.parse().map_err(|_| RequestError::Request)?;
             let mut content_buf: Vec<u8> = vec![0u8; content_length];
@@ -216,7 +239,7 @@ impl From<Request> for Vec<u8> {
         let headers = req
             .headers
             .iter()
-            .map(|(k, v)| format!("{}: {}", k.to_string(), v))
+            .map(|h| format!("{}: {}", h.name.to_string(), h.value))
             .collect::<Vec<String>>()
             .join("\r\n");
 
