@@ -1,6 +1,7 @@
 #![allow(dead_code, unused_imports)]
 use crate::http::address::Address;
-use crate::http::headers::RequestHeader;
+use crate::http::cookie::Cookie;
+use crate::http::headers::{Header, HeaderType, Headers};
 use crate::http::method::Method;
 use crate::http::Request;
 use crate::tests::mock_stream::MockStream;
@@ -16,8 +17,6 @@ fn test_request_from_stream() {
     let mut stream = MockStream::with_data(VecDeque::from_iter(test_data.iter().cloned()));
     let request = Request::from_stream(&mut stream, "1.2.3.4:5678".parse().unwrap());
 
-    assert!(request.is_ok());
-
     let request = request.unwrap();
     let expected_uri: String = "/testpath".into();
     let expected_query: String = "foo=bar".into();
@@ -28,11 +27,26 @@ fn test_request_from_stream() {
     assert_eq!(request.content, None);
     assert_eq!(request.address, Address::new("1.2.3.4:5678").unwrap());
 
-    let mut expected_headers: BTreeMap<RequestHeader, String> = BTreeMap::new();
-    expected_headers.insert(RequestHeader::Host, "localhost".to_string());
+    let mut expected_headers: Headers = Headers::new();
+    expected_headers.add(HeaderType::Host, "localhost");
     assert_eq!(request.headers, expected_headers);
 }
 
+#[test]
+fn test_cookie_request() {
+    let test_data = b"GET / HTTP/1.1\r\nHost: localhost\r\nCookie: foo=bar; baz=qux\r\n\r\n";
+    let mut stream = MockStream::with_data(VecDeque::from_iter(test_data.iter().cloned()));
+    let request = Request::from_stream(&mut stream, "1.2.3.4:5678".parse().unwrap()).unwrap();
+
+    let mut expected_cookies = vec![Cookie::new("foo", "bar"), Cookie::new("baz", "qux")];
+
+    assert_eq!(request.get_cookies(), expected_cookies);
+
+    assert_eq!(request.get_cookie("baz"), expected_cookies.pop());
+    assert_eq!(request.get_cookie("foo"), expected_cookies.pop());
+    assert_eq!(request.get_cookie("sus"), None);
+}
+
 #[test]
 fn test_bytes_from_request() {
     let mut test_data = Request {
@@ -40,17 +54,13 @@ fn test_bytes_from_request() {
         uri: "/test".into(),
         query: "foo=bar".into(),
         version: "HTTP/1.1".into(),
-        headers: BTreeMap::new(),
+        headers: Headers::new(),
         content: Some(b"this is a test".to_vec()),
         address: Address::new("1.2.3.4:5678").unwrap(),
     };
 
-    test_data
-        .headers
-        .insert(RequestHeader::ContentLength, "14".into());
-    test_data
-        .headers
-        .insert(RequestHeader::ContentType, "text/plain".into());
+    test_data.headers.add(HeaderType::ContentLength, "14");
+    test_data.headers.add(HeaderType::ContentType, "text/plain");
 
     let expected_bytes = b"GET /test?foo=bar HTTP/1.1\r\nContent-Length: 14\r\nContent-Type: text/plain\r\n\r\nthis is a test".to_vec();
 
@@ -66,8 +76,6 @@ fn test_proxied_request_from_stream() {
     let mut stream = MockStream::with_data(VecDeque::from_iter(test_data.iter().cloned()));
     let request = Request::from_stream(&mut stream, "1.2.3.4:5678".parse().unwrap());
 
-    assert!(request.is_ok());
-
     let request = request.unwrap();
     let expected_uri: String = "/testpath".into();
     assert_eq!(request.method, Method::Get);
@@ -83,13 +91,9 @@ fn test_proxied_request_from_stream() {
         }
     );
 
-    let mut expected_headers: BTreeMap<RequestHeader, String> = BTreeMap::new();
-    expected_headers.insert(RequestHeader::Host, "localhost".to_string());
-    expected_headers.insert(
-        RequestHeader::Custom {
-            name: "x-forwarded-for".into(),
-        },
-        "9.10.11.12,13.14.15.16".to_string(),
-    );
+    let mut expected_headers: Headers = Headers::new();
+    expected_headers.add(HeaderType::Host, "localhost");
+    expected_headers.add("X-Forwarded-For", "9.10.11.12,13.14.15.16");
+
     assert_eq!(request.headers, expected_headers);
 }
